from vlf4ions.flag import flags
import numpy as np
from datetime import datetime
import datetime as dt
import pytz
import os
from vlf4ions.class_definition import receiver

dirname = os.path.dirname(__file__)


def test_flags():

    # Synthetic data
    amp_NS = np.arange(1, 6, 1)
    amp_EW = amp_NS
    phase = amp_NS

    Nancay = receiver("NC", lat=47.3755933, lon=2.1944333, threshold=0.75)

    # Test in normal time
    previous_nb_points = 4
    today = dt.datetime(2000, 1, 1, 12, 3, 0, 0, pytz.UTC)
    path_to_files = os.path.join(dirname, "Data/DHO_sunrise_sunset.txt")

    flag_1, data_length_1, data_1 = flags(
        amp_NS, amp_EW, previous_nb_points, today, path_to_files, Nancay, phase
    )

    # Test random phase
    phase_disturbed = np.arange(1, 400, 30)
    flag_phase, data_length_phase, data_phase = flags(
        amp_NS,
        amp_EW,
        previous_nb_points,
        today,
        path_to_files,
        Nancay,
        phase_disturbed,
    )

    # Expected data_1
    amp = np.sqrt(amp_NS**2 + amp_EW**2)
    data_1_exp = np.median(amp)

    # Test antenna down
    flag_2, data_length_2, data_2 = flags(
        amp_NS, amp_EW, data_length_1, today, path_to_files, Nancay, phase
    )

    # Test nighttime
    night = dt.datetime(2000, 1, 1, 23, 0, 0, 0, pytz.UTC)
    amp_NS = np.ones((120, 1)) * 30
    amp_NS[0:60] = 0
    amp_EW = np.zeros((120, 1))

    flag_3, data_length_3, data_3 = flags(
        amp_NS, amp_EW, previous_nb_points, night, path_to_files, Nancay, phase
    )

    # Test transmetter down
    Nancay.threshold = 400
    flag_4, data_length_4, data_4 = flags(
        amp_NS, amp_EW, previous_nb_points, today, path_to_files, Nancay, phase
    )

    assert (
        (flag_1 == 0)
        and (flag_2 == 1)
        and (flag_3 == 3)
        and (flag_4 == 2)
        and (data_1 == data_1_exp)
        and (data_4 == 30)
        and (flag_phase == 2)
    )



import numpy as np
import vlf4ions.compute_electron_density as ced
import vlf4ions.class_definition as cd
import os

dirname = os.path.dirname(__file__)


def test_get_quiet_hb():
    sza_1 = 0
    H_1, B_1 = ced.get_quiet_hb(sza_1)

    sza_2 = np.pi / 4
    H_2, B_2 = ced.get_quiet_hb(sza_2)

    sza_3 = 45
    H_3, B_3 = ced.get_quiet_hb(sza_3)

    assert (
        (np.abs(H_1 - 70.55) < 1e-4) and (np.abs(B_2 - 0.3668) < 1e-4) and (H_2 == H_3)
    )


def test_LMP_findminimum():

    sza = 66.7753
    GQD = cd.station(
        "GQD", 54.73, -2.88, 22.1, 24.4, ":45", DP=143.680907339525, DA=10.556409437684
    )

    path_to_files = os.path.join(dirname, "Data/")

    H, B = ced.LMP_findminimum(GQD, sza, path_to_files)

    assert (np.abs(H - 69.4) < 0.1) and (np.abs(B - 0.13) < 0.001)


def test_both_function():

    GVT = cd.station("GVT", 54.91, -3.28, 19.58, 60.16, ":42", df=0)

    sza = np.pi / 4
    H_1, B_1 = ced.get_quiet_hb(sza)
    print(H_1)

    path_to_files = os.path.join(dirname, "Data/")
    H_2, B_2 = ced.LMP_findminimum(GVT, sza, path_to_files)
    print(H_2)

    assert np.abs(H_1 - H_2) < 0.05 and np.abs(B_1 - B_2) < 0.005



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 11:47:33 2024

@author: pteysseyre
"""

#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 09:59:31 2024

@author: pteysseyre
"""

import numpy as np
from pymatreader import read_mat  # To read the mau files
import os.path  # Useful to determine if a file exist or not (to write headers in a new file)
import glob

# ======================== Read mau files ======================================


def read_Narrowband_postprocessing(
    la_date, la_station, path, file_endings=["_100A", "_100B", "_101A", "_101B"]
):
    """
    Reads the data for the station and the day asked and stores them in the output arrays
    Needs the path to the data files
    Reads the .mat files (already ended)
    Written by C. Briand on 16//10/2022
    Converted to Python by P. Teysseyre on 18/03/2024

    :param la_date: Date when the files need to be read (e.g. '2023_01_25')
    :param la_station: Transmitter call sign
    :param path: path to the files
    :param file_endings: List of string. Endings of the files for amp_NS, phase_NS, amp_EW and phase_EW in that order

    :return: -amp_NS, amplitude in the N/S direction (in the unit of the file)
        - amp_EW, amplitude in the E/W direction
        - phase_NS, phase in the N/S direction
        - phase_EW, phase in the E/W direction
        - the_time: time-array (in UT)
    """

    Cal_NS = 1
    Cal_EW = 1

    nb_errors = 0

    # amp_NS
    list_of_files = glob.glob(path + "NC*" + la_station + "_100A.mat")
    latest_file = max(list_of_files, key=os.path.getctime)

    try:
        data = read_mat(latest_file)
        amp_NS = data["data"] * Cal_NS
        start_time = (
            data["start_hour"] + data["start_minute"] / 60 + data["start_second"] / 3600
        )
    except (TypeError, ValueError):
        print("TypeError - amp_NS")
        amp_NS = 0
        nb_errors += 1
        start_time = 0

    # phase_EW
    list_of_files = glob.glob(path + "NC*" + la_station + "_101B.mat")
    latest_file = max(list_of_files, key=os.path.getctime)

    try:
        data = read_mat(latest_file)
        phase_EW = data["data"]
        start_time = (
            data["start_hour"] + data["start_minute"] / 60 + data["start_second"] / 3600
        )
    except (TypeError, ValueError):
        print("TypeError - phase_EW")

        # If there is a pb with the phase_EW files, a solution is to take the phase_NS ones
        # phase_NS
        list_of_files = glob.glob(path + "NC*" + la_station + "_100B.mat")
        latest_file = max(list_of_files, key=os.path.getctime)

        try:
            data = read_mat(latest_file)
            phase_EW = data["data"]
            start_time = (
                data["start_hour"]
                + data["start_minute"] / 60
                + data["start_second"] / 3600
            )
        except (TypeError, ValueError):
            print("TypeError - phase_NS")
            phase_EW = 0
            nb_errors += 1
            start_time = 0

    # amp_EW
    list_of_files = glob.glob(path + "NC*" + la_station + "_101A.mat")
    latest_file = max(list_of_files, key=os.path.getctime)

    try:
        data = read_mat(latest_file)
        amp_EW = data["data"] * Cal_EW
        start_time = (
            data["start_hour"] + data["start_minute"] / 60 + data["start_second"] / 3600
        )
    except (TypeError, ValueError):
        print("TypeError - amp_EW")
        amp_EW = 0
        nb_errors += 1
        start_time = 0

    # phase_NS
    list_of_files = glob.glob(path + "NC*" + la_station + "_100B.mat")
    latest_file = max(list_of_files, key=os.path.getctime)

    try:
        data = read_mat(latest_file)
        phase_NS = data["data"]
        start_time = (
            data["start_hour"] + data["start_minute"] / 60 + data["start_second"] / 3600
        )
    except (TypeError, ValueError):
        print("TypeError - phase_NS")
        phase_NS = 0
        nb_errors += 1
        start_time = 0

    Fs = data["Fs"]
    tstep = 1 / Fs / 3600
    the_time = np.linspace(start_time, tstep * np.size(phase_EW), np.size(phase_EW))

    return (amp_NS, amp_EW, phase_NS, phase_EW, the_time)


def read_Narrowband_real_time(station, path, receiver, today):
    """
    Reads the data for the station and the day asked and stores them in the output arrays
    Reads the .mat files still being written
    Needs the path to the files
    Written by C. Briand on 16//10/2022
    Converted to Python on 18/03/2024

    :param station: 'station' class instance
    :param path: Path to the files being read
    :param receiver: Receiver class instance
    :param today: Datetime, time of the file reading

    :return: -amp_NS, amplitude in the N/S direction (in pT)
        - amp_EW, amplitude in the E/W direction (in pT)
        - phase_NS, phase in the N/S direction (in °, not corrected)
        - phase_EW, phase in the E/W direction (in °, not corrected)
        - the_time: time-array (in UT)
        - nb_error: Number of reading error (for flagging)"""

    nb_errors = 0

    # amp_NS
    list_of_files = glob.glob(
        path + "*" + station.name + receiver.file_endings[0] + ".mau"
    )
    latest_file = max(list_of_files, key=os.path.getctime)
    # fname = shutil.copy2(f, pathdest) # If we want to copy the files, decomment the line and change the following f to fname
    try:
        data = read_mat(latest_file)
        amp_NS = data["data"] * station.Cal_NS
    except (TypeError, ValueError):
        print("TypeError - amp_NS")
        amp_NS = 0
        nb_errors += 1
    # os.remove(fname)

    # phase_EW
    list_of_files = glob.glob(
        path + "*" + station.name + receiver.file_endings[3] + ".mau"
    )
    latest_file = max(list_of_files, key=os.path.getctime)
    # fname = shutil.copy2(f, pathdest)
    try:
        data = read_mat(latest_file)
        phase_EW = data["data"]
    except (TypeError, ValueError):
        print("TypeError - phase_EW")
        phase_EW = 0
        nb_errors += 1

    # amp_EW
    list_of_files = glob.glob(
        path + "*" + station.name + receiver.file_endings[2] + ".mau"
    )
    latest_file = max(list_of_files, key=os.path.getctime)
    # fname = shutil.copy2(f, pathdest)
    try:
        data = read_mat(latest_file)
        amp_EW = data["data"] * station.Cal_EW
    except (TypeError, ValueError):
        print("TypeError - amp_EW")
        amp_EW = 0
        nb_errors += 1
        # os.remove(fname)

    # phase_NS
    list_of_files = glob.glob(
        path + "*" + station.name + receiver.file_endings[1] + ".mau"
    )
    latest_file = max(list_of_files, key=os.path.getctime)
    # fname = shutil.copy2(f, pathdest)
    try:
        data = read_mat(latest_file)
        phase_NS = data["data"]
    except (TypeError, ValueError):
        print("TypeError - phase_NS")
        phase_NS = 0
        nb_errors += 1
        # os.remove(fname)

    Fs = data["Fs"]
    tstep = 1 / Fs / 3600

    # Construction of the time array so it matches the phase array length
    end_time = today.hour + today.minute / 60 + today.second / 3600
    length_phase = np.size(phase_EW)

    start_time = end_time - tstep * length_phase
    the_time = np.linspace(start_time, end_time, length_phase)

    if len(the_time) < 1:
        print("Problem reading the phase files")

    return (amp_NS, amp_EW, phase_NS, phase_EW, the_time, nb_errors)



import os
import vlf4ions.class_definition as cd
import numpy as np

dirname = os.path.dirname(__file__)


def test_get_pdf_coeff():

    errors = []
    NRK = cd.station("NRK", 63.85, -22.47, 37.5, 22.02, ":15", df=1.4136e-4)
    path_to_files = os.path.join(dirname, "Data/")

    NRK.DP = 0
    NRK.get_pdf_coeff(path_to_files)
    if NRK.mu != -6 or NRK.sigma != 1:
        errors.append("Problem with the prior")

    NRK.DP = 10
    NRK.get_pdf_coeff(path_to_files)
    if (
        np.abs(NRK.mu + 4.66905892623818) > 1e-4
        or np.abs(NRK.sigma - 0.199174407925015) > 1e-4
    ):
        errors.append("Problem with the first row")

    NRK.DP = 200
    NRK.get_pdf_coeff(path_to_files)
    if (
        np.abs(NRK.mu + 4.10760899331066) > 1e-4
        or np.abs(NRK.sigma - 0.481437593395161) > 1e-4
    ):
        errors.append("Problem with the last row")

    # assert no error message has been registered, else print messages
    assert not errors, "errors occured:\n{}".format("\n".join(errors))



"""Computes the polarisation parameters to avoid correcting the phase"""

import numpy as np


def EllipseParams(amp_NS, amp_EW, phase_NS, phase_EW):
    """Computes the polarisation parameters
    From Gross et al, 2018 et their Matlab code sent in private communication

    :param amp_NS, amp_EW: Amplitude in the NS and EW direction
    :param phase_NS, phase_EW: Phase in the NS and EW direction

    :returns: Phi_start, start phase. The other parameters are computed, but only this one is needed
    """

    # Delete the last value of some of the arrays so that everything has the same length
    # (This problem arises when an additional antenna measurement is made between the time when the first file is read and the last one)
    min_length = np.min(
        [np.size(amp_NS), np.size(amp_EW), np.size(phase_NS), np.size(phase_EW)]
    )
    amp_NS = amp_NS[0:min_length]
    amp_EW = amp_EW[0:min_length]
    phase_NS = phase_NS[0:min_length]
    phase_EW = phase_EW[0:min_length]

    # 1 - Compute the declination angle from transmitter to geomagnetic north
    # To do this at first approach, tan(theta) = amp_NS/amp_EW
    theta = np.arctan(amp_EW[-1] / amp_NS[-1])  # in radians

    # Convert to complex quantities
    b_NS = amp_NS * np.exp(1j * phase_NS * np.pi / 180)
    b_EW = amp_EW * np.exp(1j * phase_EW * np.pi / 180)

    # Rotate the data so that it points towards the transmitter
    R = np.zeros((2, 2))
    R[0, 0] = np.cos(theta + np.pi / 2)
    R[0, 1] = np.sin(theta + np.pi / 2)
    R[1, 0] = -np.sin(theta + np.pi / 2)
    R[1, 1] = np.cos(theta + +np.pi / 2)
    rot_xy = R @ np.vstack((b_NS, -b_EW))
    by = rot_xy[0, :]  # Br
    bx = rot_xy[1, :]  # Btheta

    # Compute ellipse parameters

    # Convert bx & by to real quantities
    bx_abs = np.abs(bx)
    by_abs = np.abs(by)
    bx_phi = np.angle(bx)
    by_phi = np.angle(by)

    # Equations 6 to 8 of Gross et al., 2018
    gamma = by_abs / bx_abs
    phi_0 = by_phi - bx_phi  # Angle between the two vectors
    tau_rad = 0.5 * np.arctan((2 * gamma) / (1 - gamma**2) * np.cos(phi_0))
    # Convert tau_rad to be correct (uncertainty on arctan)
    i_convert = by_abs > bx_abs
    tau_rad[i_convert] = tau_rad[i_convert] - np.sign(tau_rad[i_convert]) * np.pi / 2
    tau = tau_rad * 180 / np.pi  # Degrees

    # Ellipticity angle (Equation 9)
    chi = 0.5 * np.arcsin(2 * gamma / (1 + gamma**2) * np.sin(phi_0))
    chi = chi * 180 / np.pi  # Degrees

    # B_major and B_minor (Equation 11)
    b_major = by * (np.sin(tau_rad)) + bx * np.cos(tau_rad)
    b_minor = -np.sin(tau_rad) * bx + np.cos(tau_rad) * by

    # Start phase (Equation 12)
    start_phase = np.angle(b_major)
    start_phase = start_phase * 180 / np.pi  # Degree

    return start_phase



import vlf4ions.sunrise_sunset as srst
import vlf4ions as vlf
import datetime as dt
from datetime import datetime
import os
import pytz


def test_sunrise_sunset():

    dirname = os.path.dirname(__file__)
    dirname = os.path.join(dirname, "Data/")
    la_station = "NWC"
    today = dt.datetime(2025, 5, 13, 14, 28, 22, 323128, pytz.UTC)
    path = os.path.join(dirname, la_station + "_sunrise_sunset.txt")

    sunrise, sunset = srst.get_sunrise_sunset(today, path)

    assert (sunrise == dt.time(3, 48)) and (sunset == dt.time(12, 47))


def test_check_if_daytime():

    dirname = os.path.dirname(__file__)
    dirname = os.path.join(dirname, "Data/")
    la_station = "NWC"
    path = os.path.join(dirname, la_station + "_sunrise_sunset.txt")
    today = dt.datetime(2025, 5, 13, 3, 47, 0, 0, pytz.UTC)

    sunrise, sunset = srst.get_sunrise_sunset(today, path)

    flag = 0  # List of flags, 0 if in daytime, 3 in nighttime

    # Times to check
    today = dt.datetime(2025, 5, 13, 3, 48, 0, 0, pytz.UTC)
    if not srst.check_if_daytime(today, path):
        flag += 3
    today = dt.datetime(2025, 5, 13, 3, 49, 0, 0, pytz.UTC)
    if not srst.check_if_daytime(today, path):
        flag += 3
    today = dt.datetime(2025, 5, 13, 12, 46, 0, 0, pytz.UTC)
    if not srst.check_if_daytime(today, path):
        flag += 3
    today = dt.datetime(2025, 5, 13, 12, 47, 0, 0, pytz.UTC)
    if not srst.check_if_daytime(today, path):
        flag += 3

    assert flag == 6


# Not sure I need to test the get_sza function, since it only calls the get_altitude function from the pysolar library.



import vlf4ions.analyse_change_points as acp
import numpy as np


def test_analyse_bp():

    # Common parameters to all tests
    detection_threshold = 10
    amp = 2
    the_time = np.arange(0, 5, 1)
    quiet_phase = 1
    quiet_amp = 1
    last_time = 2
    quiet_p1 = 0
    quiet_breakpoint = 2
    quiet = 1

    # Keep track of errors
    errors = []

    # Test 1 - beginning of the flare
    current_time = the_time[-1]
    current_p1 = 12
    phase = np.array([1, 1, 1, 1, 2])
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        quiet_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if quiet_now == 1 or DP_now != 1:
        errors.append("Problem with breakpoint analysis for flare onset")

    # Prepare for test 2
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 4
    phase = np.array([1, 1, 1, 1, 2, 2])
    the_time = np.arange(0, 6, 1)

    # Test 2 - Flare maximum
    current_p1 = 5
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if quiet_now == 1 or DP_now != 1:
        errors.append("Problem with breakpoint analysis for flare maximum")

    # Prepare for test 3
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 5
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9])
    the_time = np.arange(0, 8, 1)

    # Test 3 - Flare decay
    current_p1 = -1
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if quiet_now == 1:
        errors.append(
            "Problem with breakpoint analysis for flare decay: declared as quiet"
        )
    if prev_timedecay != 5.5:
        errors.append("Problem with breakpoint analysis for flare decay: decay time")

    # Prepare for test 4
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 7
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7])
    the_time = np.arange(0, 10, 1)

    # Test 4 - Flare decay update
    current_p1 = -0.5
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if DP_now != 0.7:
        errors.append("Problem with flare decay update: wrong DP")
    if quiet_now == 1:
        errors.append(
            "Problem with breakpoint analysis for flare decay update: declared as quiet"
        )
    if prev_timedecay != 8.5:
        print(prev_timedecay)
        errors.append(
            "Problem with breakpoint analysis for flare decay update: decay time"
        )

    # Prepare for test 5
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 9
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3])
    the_time = np.arange(0, 12, 1)

    # Test 5 - Back to quiet times
    current_p1 = 1
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if DP_now != 0:
        errors.append("Problem with bp analysis - back to quiet: DP is not zero")
    if quiet_now == 0:
        errors.append("Problem with bp analysis - does not go back to quiet ")

    # Test 6 - Quiet after quiet
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 9
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3, 1])
    the_time = np.arange(0, 13, 1)
    current_p1 = 0
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if DP_now != 0:
        errors.append("Problem with bp analysis - quiet after quiet, DP not zeros")
    if quiet_now == 0:
        errors.append("Problem with bp analysis - does not go stay quiet ")

    # Test 7 - Quiet after quiet, with negative slope
    last_p1 = current_p1
    quiet = quiet_now
    last_time = 9
    phase = np.array([1, 1, 1, 1, 2, 2, 1.9, 1.9, 1.7, 1.7, 1.3, 1.3, 1, 1])
    the_time = np.arange(0, 14, 1)
    current_p1 = -1
    current_time = the_time[-1]
    quiet_now, prev_timedecay, DP_now, DA = acp.analyse_breakpoint(
        current_time,
        current_p1,
        amp,
        phase,
        the_time,
        quiet,
        quiet_phase,
        quiet_amp,
        last_time,
        last_p1,
        detection_threshold,
        quiet_p1,
        quiet_breakpoint,
    )
    if DP_now != 0:
        errors.append(
            "Problem with bp analysis - quiet after quiet, DP not zero if slope is negative"
        )
    if quiet_now == 0:
        errors.append(
            "Problem with bp analysis - does not go stay quiet if new slope is negative"
        )

    # assert no error message has been registered, else print messages
    assert not errors, "errors occured:\n{}".format("\n".join(errors))



# From probas computations
from scipy.stats import norm
from scipy.integrate import cumulative_simpson

# General
import numpy as np
import matplotlib.pyplot as plt
import datetime as dt
from datetime import datetime

# To compute electron density
import vlf4ions.compute_electron_density as ced
import vlf4ions.plot_map as pm
import vlf4ions.sunrise_sunset as srss

# To write result in file
import vlf4ions.manage_statefile as ms


# --------------------- Plots ------------------


def plot_probas(probas, flux, path_to_results, today, stations_on):
    """Plots the flux estimation from the values of DP. Returns the values above which there
    a 10%, 50% and 75% chance that the flux is.

    :param probas: probability density function estimated from the station DP
    :param flux: Defined internally, possible flux values (from 1e-6 to 1e-3 W/m^2)
    :param path_to_results: path to the place where the plot will be stored
    :type path_to_results: string
    :param today: Date and time of the call to the function (defined internally)
    :type today: datetime
    :param stations_on: Names of the stations which are transmitting in daytime
    :type stations_on: string

    :return: flux above which there is a 10%, 50% or 75% chance of the flux value being

    """

    ## Total probas
    integral = cumulative_simpson(probas)
    tenpercent = np.argmin(np.abs(integral / integral[-1] - 0.90))
    fiftypercent = np.argmin(np.abs(integral / integral[-1] - 0.50))
    seventyfive = np.argmin(np.abs(integral / integral[-1] - 0.25))
    best_guess = np.argmax(probas)

    flux = 10**flux

    # Prepare plot
    plt.rcParams.update({"font.size": 15})

    plt.figure()
    # plt.axvline(1e-5, linewidth=2, ls=':', color='k')
    # plt.axvline(1e-4, linewidth=2, ls=':', color='k')
    plt.plot(flux, probas, linewidth=3, color="#24492E")

    plt.axvline(
        flux[best_guess], color="#24492E", label="Best guess: %1.1e" % flux[best_guess]
    )
    plt.axvline(
        flux[tenpercent],
        color="#E69B99FF",
        label="P(flux > %1.1e) = 0.10" % flux[tenpercent],
    )
    plt.axvline(
        flux[fiftypercent],
        color="#BA7999FF",
        label="P(flux > %1.1e) = 0.50" % flux[fiftypercent],
    )
    plt.axvline(
        flux[seventyfive],
        color="#89689DFF",
        label="P(flux > %1.1e) = 0.75" % flux[seventyfive],
    )

    plt.fill_between(
        flux[seventyfive:-1], probas[seventyfive:-1], color="#89689D", alpha=0.5
    )
    plt.fill_between(
        flux[fiftypercent:-1], probas[fiftypercent:-1], color="#BA7999FF", alpha=0.5
    )
    plt.fill_between(
        flux[tenpercent:-1], probas[tenpercent:-1], color="#E69B99FF", alpha=0.5
    )

    plt.legend(loc='best', fontsize=10)
    plt.xscale("log")
    plt.xlabel("Flux (W/m^2)")
    plt.ylabel("Probability (a.u.)")
    plt.suptitle(today.strftime("%d/%m/%y %H:%M:%S"))
    plt.title(stations_on)
    plt.savefig(path_to_results + "last.pdf")
    plt.close()

    return (flux[tenpercent], flux[fiftypercent], flux[seventyfive], flux[best_guess])


# --------------------- Probabilities and forecast -------------------


class nowcast:
    """Looks at the state of all received stations, and estimate the X-flux from the Sun.
    If needed, sends alerts to specified recipients.

    :param stations: List of station class instances
    :param receiver: Receiver class instance
    :param alerts: List of potential alerts to send. Each alert may have different
     sender, recipients or trigger.
    :param reading_time: Second after the minute when the nowcast is done. It must not
     interfere with the times when the reading_time of each station. Default: ':00'.
    :param ten: Flux value above which there is a ten percent probability for the flux estimation
    :param fifty: Flux value above which there is a fifty percent probability for the flux estimation
    :param seventyfive: Flux value above which there is a seventy five percent probability for the flux estimation
    :param debug: If it is 1, show the values of DP for each station each time it is called. If it is 2, also show there flags
    """

    def __init__(
        self,
        stations,
        receiver,
        alerts,
        reading_time=":00",
        ten=6.6e-06,
        fifty=2.2e-06,
        seventyfive=1.4e-06,
        debug=0,
    ):

        self.stations = stations  # Stations to consider
        self.receiver = receiver
        self.alerts = alerts
        self.reading_time = reading_time
        self.ten = ten
        self.fifty = fifty
        self.seventyfive = seventyfive
        self.nb_stations_on = len(self.stations)
        self.debug = debug

    def run(
        self, path_to_probas, path_to_results, path_to_CSVfiles
    ):  # Compute flux estimate
        """Compute flux estimate and electron density over each path and plots them

        :param path_to_probas: path to the files where the mu/sigma values are stored for the probability computation
        :param path_to_results: path where the plots will be stored
        :param path_to_CSVfiles: path to the results from LMP"""

        today = dt.datetime.now(
            dt.timezone.utc
        )  # The time zone is necessary to use get_sza later

        flux = np.linspace(-6, -3, 1000).T

        # Uniform prior
        prior = np.ones((len(flux)))
        probas = prior

        # We keep track of stations which are on
        stations_on = ""
        nb_stations_on = 0
        quiet = 0

        for station in self.stations:

            if station.flag == 0:

                nb_stations_on += 1
                quiet += station.quiet
                stations_on = stations_on + " - " + station.name

                station.get_pdf_coeff(path_to_probas)
                proba_station = np.exp(-(((flux - station.mu) / station.sigma) ** 2))
                probas = probas * proba_station / prior

                if self.debug > 0:
                    # For a quick visual check
                    print(station.name + " -  DP: " + str(station.DP) + " °")

                # Compute Ne
                sza = srss.get_sza(today, station.lat, station.lon)
                H, B = ced.LMP_findminimum(station, sza, path_to_CSVfiles)
                station.Ne = ced.compute_Ne(70, H, B)

            if self.debug == 1:

                # For a quick visual check
                print(station.name + " -  Flag: " + str(station.flag))

        ten, fifty, seventyfive, best_guess = plot_probas(
            probas, flux, path_to_results, today, stations_on
        )
        pm.plot_map(self.stations, self.receiver, today, path_to_results)

        if (
            nb_stations_on > 1
        ):  # If there is no station, or only one station working, no alerts are sent

            # If this is the case for the first time, the user is warned
            if self.nb_stations_on == 1:

                for alert in self.alerts:
                    if alert.threshold == 0:
                        old_body = alert.body
                        old_subject = alert.subject
                        alert.body = "At least one transmitter is now usable - Flare detection is now possible again"
                        alert.subject = self.receiver.name + " - Working"
                        alert.send_email()
                        alert.body = old_body
                        alert.subject = old_subject
                        alert.sent_last_time = 0

            quiet = quiet / nb_stations_on

            # Write everything in a file
            ms.write_fluxestimate(
                best_guess, quiet, today, path_to_results, "nowcast", nb_stations_on
            )
            if self.debug == 2:
                print(quiet)
                print(best_guess)

            for alert in self.alerts:

                if alert.threshold > 0:
                    if ten > alert.threshold and alert.sent_last_time == 0:

                        # This is the beginning of the warning
                        alert.send_email()

                    elif ten < alert.threshold and alert.sent_last_time == 1:

                        # This is its end
                        old_body = alert.body
                        alert.body = "End of alert"
                        alert.send_email()
                        alert.body = old_body
                        alert.sent_last_time = 0

                    elif (
                        alert.send_each_change
                        and (
                            np.abs(ten - self.ten) / self.ten > 0.1
                            or np.abs(fifty - self.fifty) / self.fifty > 0.1
                            or np.abs(seventyfive - self.seventyfive) / self.seventyfive
                            > 0.1
                        )
                        and self.nb_stations_on == nb_stations_on
                        and quiet < 1
                        and ten > alert.threshold
                    ):

                        # This is sent each time the flux estimation changes
                        alert.send_email()
                        self.ten = ten
                        self.fifty = fifty
                        self.seventyfive = seventyfive

        else:  # We need to warn people that no stations are usable at this point

            for alert in self.alerts:
                if alert.threshold == 0 and alert.sent_last_time == 0:
                    old_body = alert.body
                    old_subject = alert.subject
                    alert.body = "All transmitters are off or in nighttime - No flare detection is possible"
                    alert.subject = self.receiver.name + " - On hold"
                    alert.send_email()
                    alert.body = old_body
                    alert.subject = old_subject
                    alert.sent_last_time = 1

                # Make sure to signal internally end of alert (do not send emails)
                elif ten < alert.threshold and alert.sent_last_time == 1:
                    alert.send_last_time = 0

        self.nb_stations_on = nb_stations_on

        print("Nowcast - ", today.strftime("%H:%M:%S"), " - Done")



import vlf4ions.polarisation as polar
import numpy as np


def test_ellipseparams():

    # Synthetic data (matches the tests by Gross et al, although we flip their start phase to match the measured phase sign)
    # Without Nans because we don't have them in the data in real time
    amp_NS = np.array([1, 0.1])
    amp_EW = amp_NS
    phase_NS = np.array([0, 45])
    phase_EW = np.array([5, 53])

    start_phase = polar.EllipseParams(amp_NS, amp_EW, phase_NS, phase_EW)
    print(start_phase)

    assert (
        np.max(np.abs(start_phase + np.array([180, 127]))) < 5
    )  # We allow 5° because our declination is 45° and theirs 20°



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:41:33 2024

@author: pteysseyre
"""

import numpy as np
import pandas as pd  # NOTE : we only need this to read csv files; there may be an easier way to do this, though that works too
import vlf4ions.sunrise_sunset as srst


def get_quiet_hb(sza):
    """Returns an estimate of the quiet values for h' and beta (Appendice A, MacRae & Thomson, 2000)

    :param sza: Solar zenith angle (in rad)

    :return: - H_quiet: quiet value of h'
        - B_quiet: quiet value of beta"""

    # Check if the value is in rad
    if sza > 2 * np.pi or sza < -2 * np.pi:
        sza = sza * np.pi / 180

    H_quiet = (
        70.55
        + 0.0045 * sza
        + 1.270 * sza**2
        - 0.774 * sza**3
        + 3.115 * sza**4
        + 0.595 * sza**5
        - 1.491 * sza**6
        - 0.122 * sza**7
        + 0.268 * sza**8
    )
    B_quiet = (
        0.395
        + 0.005 * sza
        - 0.043 * sza**2
        + 0.0075 * sza**3
        - 0.01991 * sza**4
        - 0.0054 * sza**5
    )
    return (H_quiet, B_quiet)


def LMP_findminimum(station, sza, pathtoCSVfiles):
    """Gives the time profile of H, B. NOTE : Make sure that the relevant csv files are stored on the computer AND that they have been detrended
        They should contain A/P values, with h and beta values in their column/rows. Lastly, because of the detrend, the modelled phase may be shifted by a multiple of 90°

    :param station: station class instance
    :param sza: Solar zenith angle at the middle of the path
    :param pathtoCSVfiles: path to the files were the LMP results are stored

    :return: H, B arrays of h' and beta values

    History : written in march 2024 by P.Teysseyre
    In May 2025, reworked to use DA/DP
    Based on golden ratio method"""

    la_station = station.name

    # Read the files for modelled amplitude and phase
    # DO NOT open the files with Numbers before !
    A = pd.read_csv(pathtoCSVfiles + "Amplitude" + la_station + ".csv", header=None)
    P = pd.read_csv(pathtoCSVfiles + "Phase" + la_station + ".csv", header=None)

    A = A.to_numpy()
    P = P.to_numpy()
    A = A.T
    P = P.T

    # Get the ranges of parameters H' and beta
    h = A[0, 1:]
    b = A[1:, 0]

    # Get the correct shape for A and P
    A = A[1:, 1:]
    P = P[1:, 1:]

    # Find the value of DA/DP which would have corresponded to the background
    H_quiet, B_quiet = get_quiet_hb(sza)
    id_h = np.argmin(np.abs(h - H_quiet))
    id_b = np.argmin(np.abs(B_quiet - b))
    A_quiet = A[id_b, id_h]
    P_quiet = P[id_b, id_h]

    # Correct the values of DA and DP so that they represent total A and P
    Amp = station.DA + A_quiet
    Phase = station.DP + P_quiet

    # Reshape the data
    data = np.array([Amp, Phase])

    # ----------------------------
    # From that point onwards, the code compares the data and the modelled amp and phase to get H' and beta

    (rows, columns) = np.shape(A)

    # We remove all the NaNs
    P = P[~np.isnan(A)]
    A = A[~np.isnan(A)]

    # Compute the matrices of relative errors:
    Aq = np.abs(A - data[0]) / (data[0] + 1e-2)
    Pq = np.abs(P - data[1]) / (data[1] + 1e-2)

    # We normalise the matrices (otherwise the errors on the phase are given too much weight
    # as the phase has a much higher range than the amplitude)
    A_new = (Aq - np.min(Aq)) / (np.max(Aq) - np.min(Aq))
    P_new = (Pq - np.min(Pq)) / (np.max(Pq) - np.min(Pq))

    # The aim is to find eps such that only one point has an error in phase and amp less than eps

    # For the golden ratio method, four points are necessary:
    #   - the upper point will be eps_up, with corresponding matrices A_new, P_new and rc
    #   - then eps_3, with A_new3, P_new3, rc3 and rc_new3
    #   - then eps_4, with A_new4, P_new4, rc4, rc_new4
    #   - lastly eps_down, with no corresponding matrix

    # A_new's and P_new's are the amplitude and phase of the points with relative

    # Loop counter:
    i = 0

    # Golden ratio number
    r = 1 / ((1 + np.sqrt(5)) / 2)

    # Initialisation of the interations
    eps_up = np.max([np.max(A_new), np.max(P_new)])
    eps_down = 0
    interval = eps_up - eps_down
    eps_4 = r**2 * interval + eps_down
    eps_3 = r * interval + eps_down

    # Initialisation of rc (Technically, only the last line is absolutely necessary. The lines before just flatten A_new and P_new)
    rc = (A_new <= eps_up) & (np.abs(P_new) <= eps_up)
    A_new = A_new[rc]
    P_new = P_new[rc]
    rc = np.arange(np.size(A_new))

    # Initialisation of A_new4, P_new4 and rc4
    rc_new4 = (A_new < eps_4) & (np.abs(P_new) < eps_4)
    A_new4 = A_new[rc_new4]
    P_new4 = P_new[rc_new4]
    rc4 = rc[rc_new4]

    # Initialisation of A_new3, P_new3 and rc3
    rc_new3 = (A_new < eps_3) & (np.abs(P_new) < eps_3)
    A_new3 = A_new[rc_new3]
    P_new3 = P_new[rc_new3]
    rc3 = rc[rc_new3]

    while ((np.size(rc3) != 1) or (np.size(rc4) != 1)) and (i < 100):
        i += 1

        if np.size(A_new4) == 0:  # the new interval will be from eps_4 to eps_up
            eps_down = eps_4

            eps_4 = eps_3
            A_new4 = A_new3
            P_new4 = P_new3
            rc4 = rc3

            interval = eps_up - eps_down

            # Recompute eps_3, A_new3, P_new3 and rc_3
            eps_3 = r * interval + eps_down
            rc_3 = (A_new < eps_3) & (np.abs(P_new) < eps_3)
            A_new3 = A_new[rc_3]
            P_new3 = P_new[rc_3]

        else:  # New interval will be from eps_down to eps_3
            eps_up = eps_3
            A_new = A_new3
            P_new = P_new3
            rc = rc3

            eps_3 = eps_4
            A_new3 = A_new4
            P_new3 = P_new4
            rc3 = rc4

            interval = eps_up - eps_down

            # Recompute eps_4, A_new4, P_new4 and rc_4
            eps_4 = r**2 * interval + eps_down
            rc_new4 = (A_new3 < eps_4) & (np.abs(P_new3) < eps_4)
            A_new4 = A_new3[rc_new4]
            P_new4 = P_new3[rc_new4]
            rc4 = rc3[rc_new4]

    if np.size(A_new4) == 1:
        rc = rc4
    else:
        rc = rc3

    if (
        np.size(rc) != 1
    ):  # if the loop as not converge in 100 iterations, NaN values are kept
        H = float("nan")
        B = float("nan")
    else:
        row, col = np.unravel_index(rc[0], (rows, columns))
        H = h[col]
        B = b[row]
    return (H, B)


def compute_Ne(z, H, B):
    """Applies Wait's profile to H and B to obtain Ne at altitude z

    :param z: Height of interest in the ionosphere (in km)
    :param H: h' values (in km)
    :param B: beta value (in km-1)

    :return: Electron density (in m^-3) at altitude z"""

    Ne = 1.43e13 * np.exp(-0.15 * H) * np.exp((B - 0.15) * (z - H))

    return Ne



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Aug 27 11:42:43 2024

@author: pteysseyre
"""

import numpy as np


def find_break_point_realtime(the_time, data, delta=0.1):
    """
    Based on the incremental algorithm presented by Guralnik & Srivastava (1999)

    :param the_time: time-array
    :param data: the phase data that has just been read (time-averaged & smoothed if needed)
    :param delta: in percent/100 (default : 0.1). This is the threshold over which we
     decide that a breakpoint is indeed a breakpoint and not due to noise. In
     Guralnik & Srivastava's paper, this is also called delta and only appears
     for the incremental algorithm.

    :return: - detec: Boolean, 'True' if a breakpoint was detected, 'False' if not
     - p1: Slope of the detected breakpoint

    """

    L = len(data)

    if L > 9:  # At least 9 points are necessary for this

        # Compute likelihood assuming no breakpoints:
        likelihood_without_bp, n = _compute_likelihood_criteria(0, L, the_time, data)

        # Compute the likelihood criteria, assuming a breakpoint 5 points ago:
        likelihood_with_bp, p1 = _compute_likelihood_criteria_several_change_points(
            the_time, data, np.array([0, L - 6, L - 1])
        )

        # Determine whether there is a new breakpoint
        detec = (
            likelihood_without_bp - likelihood_with_bp
        ) / likelihood_without_bp > delta

    else:

        detec = False
        p1 = 0

    return detec, p1


def detect_change_points(the_time, data, threshold):
    """Based on Guralnik, V. & Srivastava, J. (1999)
    Detect change_points in the 'data' array in post_processing (bash algorithm)

    :param the_time: time-array
    :param data: Data array
    :param threshold: Criterion used to end the search for breakpoints. (s in the article)

    :return: change_points: time of breakpoints found in the data array"""

    # Initialisation
    change_points = np.array([0, len(data)])
    likelihood, dummy = _compute_likelihood_criteria(
        change_points[0], change_points[1], the_time, data
    )
    last_likelihood = likelihood
    stopping_criterion = 1
    loop_counter = 0

    while stopping_criterion > threshold and loop_counter < 1e3:

        loop_counter += 1
        N = len(change_points) - 1  # Number of segments

        for s in range(0, N):

            if change_points[s + 1] - change_points[s] > 5:
                new_candidate = _find_candidate_detection(
                    change_points[s], change_points[s + 1], the_time, data
                )
                temp_change_points = np.insert(change_points, s + 1, new_candidate)
                new_likelihood_criterion = (
                    _compute_likelihood_criteria_several_change_points(
                        the_time, data, temp_change_points
                    )
                )

                if new_likelihood_criterion < likelihood:
                    likelihood = new_likelihood_criterion
                    new_change_points = temp_change_points

        # Prepare for next iteration
        change_points = new_change_points
        stopping_criterion = (last_likelihood - likelihood) / last_likelihood

    return change_points


def _compute_likelihood_criteria_several_change_points(the_time, data, change_points):
    """Based on Guralnik, V., & Srivastava, J. (1999). Computes the total likelihood criterion for data,
     knowing that there are change_points at specified locations

    :param the_time: time-array
    :param data: Data in which to look for change points
    :param change_points: list of indices of the changepoints already computed.
     The first index is 0 (for the beggining of the 'data' array), the last is
     the number of elements in the 'data' array.

    :return: - total_L: total likelihood criterion
    - a: slope associated with the last breakpoint"""

    N = len(change_points) - 1
    total_L = 0

    for k in range(0, N):
        likelihood_criteria, a = _compute_likelihood_criteria(
            change_points[k], change_points[k + 1], the_time, data
        )
        total_L += likelihood_criteria

    return total_L, a


def _find_candidate_detection(i, j, the_time, data):
    """Based on Guralnik, V., & Srivastava, J. (1999).,

    Finds the best candidate to be a change-point in the array data[i:j].

    :param i: indice of the start of the interval of interest
    :param j: indice of the end of the interval
    :param the_time: time-array
    :param data: data array

    :return: the indice in the array data (! not data[i:j]!) of this best candidate.
    """

    optimal_likelihood_criteria = 1e14  # Minimum likelihood criteria. The point with the lowest likelihood criteria will be the best candidate
    split = 0

    for k in range(i + 2, j - 2):

        likelihood_first_segment, dummy = _compute_likelihood_criteria(
            i, k, the_time, data
        )
        likelihood_second_segment, dummy = _compute_likelihood_criteria(
            k + 1, j, the_time, data
        )
        likelihood_all_segments = likelihood_first_segment + likelihood_second_segment

        if likelihood_all_segments < optimal_likelihood_criteria:
            optimal_likelihood_criteria = likelihood_all_segments
            split = k

    return split


def _compute_likelihood_criteria(i, j, the_time, data):
    """Computes the likelihood criteria assuming an heteroscedastic error model.
    Based on Guralnik, V., & Srivastava, J. (1999, August). Event detection from time series data. In Proceedings of the fifth ACM SIGKDD international conference on Knowledge discovery and data mining (pp. 33-42).

    :param i: indice of the start of the interval of interest
    :param j: indice of the end of the interval
    :param the_time: time-array
    :param data: data array

    :return: - L, Likelihood criteria on the interval
        - a, slope of the linear fit for the data"""

    m = j - i + 1  # Number of elements in the the_time & data arrays

    a, b = np.polyfit(
        the_time[i : j + 1], data[i : j + 1], 1
    )  # Use a linear model to fit the segment. THIS CAN BE CHANGED !

    rss = np.sum(np.square(a * the_time[i : j + 1] + b - data[i : j + 1]))

    L = m * np.log(rss / m)

    return L, a



import vlf4ions.manage_statefile as ms
import datetime as dt
from vlf4ions.class_definition import station
from datetime import datetime, timedelta
import os
import pandas as pd
import pytz
import glob

dirname = os.path.dirname(__file__)


def test_write_fluxestimation():

    today = dt.datetime(2000, 1, 1, 12, 3, 0, 0, pytz.UTC)
    path_to_files = os.path.join(dirname, "Results/")
    filename = path_to_files + "2000_01_01_test.csv"

    # Remove file if it exists
    if os.path.exists(filename):
        os.remove(filename)

    # Test file creation
    ms.write_fluxestimate(1e-5, 0, today, path_to_files, "test", 3)

    # Test adding new data
    today = dt.datetime(2000, 1, 1, 12, 5, 0, 0, pytz.UTC)
    ms.write_fluxestimate(1e-6, 1, today, path_to_files, "test", 4)

    # Read the file
    df = pd.read_csv(path_to_files + "2000_01_01_test.csv")
    first_line = df.iloc[0]
    second_line = df.iloc[1]

    assert (
        df.size == 8
        and first_line["Time (UT)"] == 12.05
        and second_line["Quiet"] == 1
        and second_line["Number of stations"] + first_line["Number of stations"] == 7
    )


def test_write_df():

    NRK = station("NRK", 56.2, -7.6, 37.5, 22.02, ":10", df=1.4136e-4)
    NAA = station("NAA", 51.6, -33.5, 24.0, 32.39, ":18", df=3.1128e-5)
    stations = [NRK, NAA]

    path_to_results = os.path.join(dirname, "Results/")

    # Remove all previous files
    filename = glob.glob(path_to_results + "*_df.csv")
    filename = max(filename, key=os.path.getctime)  # To get a filename and not a list

    # Remove file if it exists
    if os.path.exists(filename):
        os.remove(filename)

    ms.write_file_with_df(stations, path_to_results)

    # Read the file for TODAY (list of files needs to be reload)
    filename = glob.glob(path_to_results + "*_df.csv")
    filename = max(filename, key=os.path.getctime)  # To get a filename and not a list
    df = pd.read_csv(filename)
    first_line = df.iloc[0]

    assert df.size == 2 and first_line["NAA"] == 3.1128e-5


def test_manage_bp():

    path = os.path.join(dirname, "Results/")

    # Remove all previous files
    filename = glob.glob(path + "*Test_ms.csv")

    # Remove file if it exists
    for f in filename:
        if os.path.exists(f):
            os.remove(f)

    # Create parameters for test
    la_station = "Test_ms"

    # Get today's date in a good format
    today = dt.datetime.now(
        dt.timezone.utc
    )  # The time zone is necessary to use get_sza later
    la_date = today.strftime("%Y_%m_%d")

    # Quiet is False
    the_time, slope, amp, phase, quiet, DP = ms.get_lastbreakpoint(
        la_station, la_date, path, check_yesterday=False
    )

    # Write new breakpoint
    ms.write_newbreakpoint(la_station, la_date, path, 2, 2, 2, 2, 0, 2)

    # Get last breakpoint
    the_time_2, slope_2, amp_2, phase_2, quiet_2, DP_2 = ms.get_lastbreakpoint(
        la_station, la_date, path, check_yesterday=False
    )

    # Get last quiet breakpoint
    the_time_1, slope_1, amp_1, phase_1, quiet_1, DP_1 = ms.get_lastbreakpoint(
        la_station, la_date, path, quiet=True, check_yesterday=False
    )

    assert (
        (the_time == 0)
        and quiet == 1
        and quiet_1 == 1
        and quiet_2 == 0
        and slope_2 == 2
        and amp_2 == 2
        and phase_1 == 0
        and DP == 0
    )


def test_midnight_crossing():

    # Specifically tests the case when a flare happens across midnight

    path = os.path.join(dirname, "Results/")

    # Remove all previous files
    filename = glob.glob(path + "*Test_midnight.csv")

    # Remove file if it exists
    for f in filename:
        if os.path.exists(f):
            os.remove(f)

    # Create parameters for test
    la_station = "Test_midnight"

    # Get today's date in a good format
    today = dt.datetime.now(dt.timezone.utc)
    la_date = today.strftime("%Y_%m_%d")

    # Get yesterday as well
    yesterday = today - dt.timedelta(days=1)
    hier = yesterday.strftime("%Y_%m_%d")

    # Create file for 'yesterday'
    the_time, slope, amp, phase, quiet, DP = ms.get_lastbreakpoint(
        la_station, hier, path, check_yesterday=False
    )

    # Add a flare breakpoint
    ms.write_newbreakpoint(la_station, hier, path, 12, 12, 0, 0, 0, 38)

    # Create file for today
    the_time, slope, amp, phase, quiet, DP = ms.get_lastbreakpoint(
        la_station, la_date, path, check_yesterday=True
    )

    # Normally, there should be two lines in the new file, with yesterday's data

    # Get last quiet breakpoint for today
    the_time_q, slope, amp, phase, quiet, DP = ms.get_lastbreakpoint(
        la_station, la_date, path, check_yesterday=False, quiet=True
    )

    # Get last breakpoint for today
    the_time, slope, amp, phase, quiet, DP = ms.get_lastbreakpoint(
        la_station, la_date, path, check_yesterday=False, quiet=False
    )

    assert the_time == -12 and the_time_q == -24



import vlf4ions.correct_phase as cp
import datetime as dt
from vlf4ions.class_definition import station
import os
import numpy as np

dirname = os.path.dirname(__file__)


def test_compute_df():

    # Required parameters
    date_inf = dt.datetime(2024, 5, 1)
    date_sup = dt.datetime(2024, 5, 5)
    NRK = station("NRK", 56.2, -7.6, 37.5, 22.02, ":10", df=1.4136e-4)
    path_to_files = os.path.join(dirname, "/Data/")
    file_endings = ["_100A", "_100B", "_101A", "_101B"]
    epsilon = 5

    df = cp.compute_df(date_inf, date_sup, NRK, path_to_files, file_endings, epsilon)

    assert np.abs(df - 1.3672e-4) < 0.1e-4



import vlf4ions.class_definition as cd
import vlf4ions.manage_statefile as ms
import numpy as np
import os

import pandas as pd
import datetime as dt
from datetime import datetime

dirname = os.path.dirname(__file__)


def test_email_alert_update_body():
    test_alert = cd.email_alerts(
        "test_subject", "test_body", "test_sender", "test_password", ["test_recipients"]
    )
    test_alert.update_detection_body("NRK", 0)
    # print(test_alert.body)
    assert test_alert.body != "test_body"


# TODO: test send_email


def test_sliding_median():
    test_array = np.arange(6)
    res = cd.sliding_median(test_array, 3)

    assert all(res == [1.0, 2.0, 3.0, 4.0])


def test_craft_newsletter():

    # Define alert
    test = cd.email_alerts(
        "Test", "Placeholder", "fakesender", "fakepassword", "fakerecipients"
    )

    path = os.path.join(dirname, "Data/Test_flux.csv")
    flux, times, starts, ends = test.craft_newsletter(path, True)

    assert (
        test.body.count("\n") == 17
        and flux == [2, 5]
        and times == [2, 6]
        and starts == [1, 5]
        and ends == [3, 7]
    )


def test_craft_newsletter_quiet():
    """Test the newsletter when there is no detection"""

    # Define alert
    test = cd.email_alerts(
        "Test", "Placeholder", "fakesender", "fakepassword", "fakerecipients"
    )

    path = os.path.join(dirname, "Data/Test_flux_quiet.csv")
    flux, times, starts, ends = test.craft_newsletter(path, True)

    assert len(flux) == 0 and test.body.count("\n") == 5 and test.body.count("!") == 1


def test_good_format_time():

    test = cd.email_alerts(
        "Test", "Placeholder", "fakesender", "fakepassword", "fakerecipients"
    )

    time_1 = 13
    time_1str = test._good_format_time(time_1)

    time_2 = 13.20
    time_2str = test._good_format_time(time_2)

    time_3 = 13.21
    time_3str = test._good_format_time(time_3)

    time_4 = 13.75
    time_4str = test._good_format_time(time_4)

    assert (
        time_1str == "13:00"
        and time_2str == "13:12"
        and time_3str == "13:13"
        and time_4str == "13:45"
    )


def test_craft_newsletter_quiet():
    """Test the newsletter when there is no detection"""

    # Define alert
    test = cd.email_alerts(
        "Test", "Placeholder", "fakesender", "fakepassword", "fakerecipients"
    )

    path = os.path.join(dirname, "Data/Test_flux_quiet.csv")
    flux, times, starts, ends = test.craft_newsletter(path, True)

    assert len(flux) == 0 and test.body.count("\n") == 5 and test.body.count("!") == 1


def test_good_format_time():

    test = cd.email_alerts(
        "Test", "Placeholder", "fakesender", "fakepassword", "fakerecipients"
    )

    time_1 = 13
    time_1str = test._good_format_time(time_1)

    time_2 = 13.20
    time_2str = test._good_format_time(time_2)

    time_3 = 13.21
    time_3str = test._good_format_time(time_3)

    time_4 = 13.75
    time_4str = test._good_format_time(time_4)

    assert (
        time_1str == "13:00"
        and time_2str == "13:12"
        and time_3str == "13:13"
        and time_4str == "13:45"
    )


def test_two_hours_data():

    # Station
    st = cd.station("Test", 0, 0, 0, 0.01, ":00")
    st.p1 = 0
    Nancay = cd.receiver("NC", lat=47.3755933, lon=2.1944333, threshold=-1)
    alert_antenna = cd.email_alerts("", "", "", "", "")

    # Paths
    path_to_data = os.path.join(dirname, "Data/")

    # Create breakpoint file (must have the date in the name, that's why we create
    # it each time we test the function)
    today = dt.datetime.now(
        dt.timezone.utc
    )  # The time zone is necessary to use get_sza later
    la_date = today.strftime("%Y_%m_%d")
    ms.get_lastbreakpoint("Test", la_date, path_to_data, check_yesterday=False)

    # Update breakpoint with synthetic data longer than two hours
    st.update_breakpoints(
        path_to_data, path_to_data, path_to_data, alert_antenna, [], Nancay
    )

    # Remove created file
    filename = path_to_data + la_date + "_Test.csv"

    # Remove file if it exists
    if os.path.exists(filename):
        os.remove(filename)

    # The slope in the test file is exactly 1 for the past 9 minutes and 0 before
    # Since the Test station has been set with a threshold of 0.01, if the code
    # works, the detected slope should be 1 (because the data is longer than two
    # hours without a breakpoint). It thus should counts as a flare detection,
    # except we actually set quiet to be 1 manually.
    # Because of all the noise I added to the synthetic files, we allow 0.5°/hr error on the slope
    assert np.abs(st.p1 - 1) < 0.5 and st.quiet == 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 22 14:10:40 2025

@author: pteysseyre
"""

import vlf4ions.read_files as rf
import vlf4ions.detect_change_points as dcp
import vlf4ions.correct_phase as correct_phase
import vlf4ions.flag as fl
import vlf4ions.analyse_change_points as acp
import vlf4ions.manage_statefile as msf
import vlf4ions.polarisation as polar

import numpy as np
import datetime as dt  # Necessary to use timezone
from datetime import datetime, timedelta, timedelta
import logging

import pandas as pd

# For alerts
import smtplib
from pathlib import Path
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.utils import COMMASPACE, formatdate
from email import encoders


# ----------------------- Receiver description -----------------
class receiver:
    """Characteristics of the receiver (for plotting purposes mainly and for sending alerts)

    :param name: Name of the receiver. Must match the name in the VLF data file names
    :param lat: Latitude of the receiver (for plotting)
    :param lon: Longitude of the receiver (for plotting)
    :param threshold: Amplitude threshold bellow which we declare a station not transmitting (default: 0.5)
    :param nb_errors: Number of reading errors, we keep track of them to alert in case of malfunction
    :param file_endings: List of strings found at the end of the data files. Default (for AWESOME): ["_100A", "_100B", "_101A", "_101B"]
    """

    def __init__(
        self,
        name,
        lat,
        lon,
        threshold=0.5,
        nb_error=0,
        file_endings=["_100A", "_100B", "_101A", "_101B"],
    ):
        self.name = name
        self.lat = lat
        self.lon = lon
        self.threshold = threshold
        self.nb_error = nb_error
        self.file_endings = file_endings


# ---------------------- Alerts, warnings and such --------------
class email_alerts:
    """Alerts sent in case of flare detection, antenna problems or such

    :param subject: Subject of the email alert
    :param body: Body of the email alert
    :param sender: Email adress of the sender
    :param password: Password of the email address of the sender
    :param recipiends: Recipients of the email alert (must be a list of strings)
    :param threshold: Flux threshold. If the probability of the flux being above this threshold
        is 10% or more, the alert will be sent (default: 0)
    :param files: List of files to include as attachment (default: empty)
    :param send_last_time: Computed internally; used to keep track of alerts already sent to avoid spamming people
    :param send_each_change: Boolean. If true, the recipients will receive an alert each time the flux estimation changes, as long
        as there is at least a 10% probability of the flux being above the threshold"""

    def __init__(
        self,
        subject,
        body,
        sender,
        password,
        recipients,
        threshold=0,
        files=[],
        sent_last_time=0,
        send_each_change=False,
    ):
        self.subject = subject
        self.body = body
        self.sender = sender
        self.password = password
        self.recipients = recipients
        self.threshold = threshold  # Only send the alerts if the proba of the flare is 10% above a threshold
        self.files = files  # Files to attach to the email
        self.sent_last_time = (
            sent_last_time  # Boolean, only useful for alerts on probas
        )
        self.send_each_change = (
            send_each_change  # Boolean, if True send each time the proba change
        )

    def update_detection_body(self, callsign, p1):
        """Just updates the body of the email with the relevant informations in case of flare detection"""

        self.body = "Detection with " + callsign + "\n Slope : " + str(p1) + " deg/hr."

    def get_path_to_fluxestimate(self, path):
        """Get the path to the file used by the 'craft_newsletter' function bellow

        :param path: Path to the flux estimate recap

        :return: filename, full name and path of the file with the flux estimations"""

        today = dt.datetime.now(dt.timezone.utc)
        if today.hour == 0:
            # This is the beginning of the day, so we want to send the recap
            # for the previous day
            today = today - dt.timedelta(days=1)

        la_date = today.strftime("%Y_%m_%d")
        filename = path + la_date + "_nowcast.csv"

        return filename

    def craft_newsletter(self, filename, completenamegiven=False):
        """Craft a newsletter based on the recapitulative file indicated by `filename'.
        It should include the highest flux estimated, its time, and each time the estimated
        flux was above M2 (with start, end time and peak times).

        :param filename: Path to the recapitulative file created with the 'write_fluxestimate' function. It can be found by calling the 'get_path_to_fileestimate' method just before
        :param completenamegiven: Boolean (default: False). If it is false, it assumes that the filename given is the path to the folder, and that the true filename must be computed (because it depends on the date)

        :return: - max_flux: List of peak fluxes for each flare detected during the day
            - max_times: List of each time of peak flux previously returned
            - starts: Start times of each detected flare (quiet < 1)
            - ends: End times of each detected flare (quiet == 1)

        """

        if not completenamegiven:
            filename = self.get_path_to_fluxestimate(filename)

        # Read the file and convert the data into a numpy array
        df = pd.read_csv(filename)
        data = df.to_numpy()

        # Convert data to convenient variables
        flux = data[:, 1]
        the_time = data[:, 0]
        quiet = data[:, 2]
        nb_stations = data[:, 3]

        # For each time when quiet is not one, get the beginning/end time and maximum estimated flux
        max_flux = []
        starts = []
        ends = []
        max_times = []

        # If quiet[0] < 1, it means that the first station in daytime has
        # detected a flare before the second station becomes in daytime
        if quiet[0] < 1:
            starts.append(the_time[0])
            i_start = 0

        # There must be a more elegant way to do this, but this works too
        for i in range(1, len(flux)):

            if quiet[i] < 1 and quiet[i - 1] == 1:  # Flare beginning
                starts.append(the_time[i])
                i_start = i
            elif (quiet[i] == 1 and quiet[i - 1] < 1) or (
                quiet[i] < 1 and i == len(flux)
            ):  # Flare ending
                ends.append(the_time[i])
                i_max = np.argmax(flux[i_start : i + 1])
                max_flux.append(flux[i_max + i_start])
                max_times.append(the_time[i_max + i_start])

        nb_flares = len(max_flux)

        # Write the body of the newletter
        body = "Today's detections : " + "\n"  # Initialisation
        body += "\n"
        body += (
            "From "
            + self._good_format_time(the_time[0])
            + " to "
            + self._good_format_time(the_time[-1])
            + " UT \n \n"
        )

        if nb_flares > 0:
            for f in range(nb_flares):
                body += "Flare " + str(f + 1) + " : \n"
                body += "-------------------- \n"
                body += "Start time : " + self._good_format_time(starts[f]) + " UT \n"
                body += (
                    "Max flux : "
                    + "{flux:.1e} W/m^2".format(flux=max_flux[f])
                    + "at "
                    + self._good_format_time(max_times[f])
                    + " UT \n"
                )
                body += "End time : " + self._good_format_time(ends[f]) + " UT \n \n"
        else:
            body += "No detection today !"

        # Reminder
        body += "\n Note: Please remember that the quality of the flux estimates depends on the data used to compute the probability functions"
        body += " If only M and weak X flares are used, the estimates will be skewed for C flares and strong X flares"
        self.body = body

        return max_flux, max_times, starts, ends

    def _good_format_time(self, the_time):
        """Convert a float like 13.1222222 to a string like '13:07'
        :param the_time: Time to convert to string
        :return: The time, converted to a string in the format '%H:%M'"""

        hour = np.floor(the_time)
        minute = np.round((the_time - hour) * 60)
        the_time = dt.time(int(hour), int(minute))

        the_time_str = the_time.strftime("%H:%M")

        return the_time_str

    def send_email(self):
        """Sends the email from the sender email adress to the recipients"""

        msg = MIMEMultipart()
        msg["From"] = self.sender
        msg["To"] = COMMASPACE.join(self.recipients)
        msg["Date"] = formatdate(localtime=True)
        msg["Subject"] = self.subject

        msg.attach(MIMEText(self.body))

        for path in self.files:
            part = MIMEBase("application", "octet-stream")
            with open(path, "rb") as file:
                part.set_payload(file.read())
            encoders.encode_base64(part)
            part.add_header(
                "Content-Disposition", "attachment; filename={}".format(Path(path).name)
            )
            msg.attach(part)

        with smtplib.SMTP_SSL("smtp.gmail.com", 465) as smtp_server:
            smtp_server.login(self.sender, self.password)
            smtp_server.sendmail(self.sender, self.recipients, msg.as_string())
            smtp_server.quit()

        self.sent_last_time = 1


# ------------------- Stations and related functions --------------


def sliding_median(arr, window):
    return np.median(np.lib.stride_tricks.sliding_window_view(arr, (window,)), axis=1)


class station:
    """Characteristics of the transmitter.

    :param name: call-sign (e.g. 'GVT')
    :param lat: latitude of the transmitter
    :param lon: latitude of the transmitter
    :param freq: Frequency (kHz)
    :param detection_threshold: Phase slope above which we detect a solar flare
    :param reading_time: second after the minute at which we read the data. NOTE: They should be unique for each station
    :param time_average: Time resolution of the smoothed signal (default: 60 s)
    :param delta: Delta parameter of the incremental algorithm (Guralnik & Srivastava, 1999. Default:0.1)
    :param nb_data_points: number of data points last time the file was read (default : 0, used for flagging)
    :param Ne: Average electron density computed in the path (not done yet, but I needed this for data analysis)
    :param DA/DP: Computed internally, variations of the amplitude and phase compared to quiet times
    :param mu/sigma: This is computed internally, and is the average and std of the flux probability density function given DA and DP
    :param orientation: Is 1 if the phase considered is E/W, 0 if it is N/S (default: 1). The idea is that the phase can be much noisier in
        one direction, so the optimal orientation is left as a choice for the user

    ---------------
    History:
        Written by P. Teysseyre in January 2025
        Re-written under current format in April 2025



    """

    def __init__(
        self,
        name,
        lat,
        lon,
        freq,
        detection_threshold,
        reading_time,
        Cal_NS=0.14,
        Cal_EW=0.08,
        df=0,
        time_average=60,
        delta=0.1,
        nb_data_points=0,
        nb_error=0,
        Ne=1e8,
        DP=0,
        DA=0,
        p1=0,
        quiet=1,
        flag=0,
        mu=-6,
        sigma=1,
    ):
        self.name = name  # Call-sign
        self.lat = lat  # Latitude
        self.lon = lon  # Longitude
        self.freq = freq  # Frequency
        self.detection_threshold = detection_threshold
        self.reading_time = reading_time  # When the data is read (must not be the same for two stations)
        self.Cal_NS = Cal_NS  # Calibration number in the N/S orientation
        self.Cal_EW = Cal_EW  # Calibration number in the E/W orientation
        self.time_average = time_average  # Time-resolution
        self.delta = (
            delta  # Delta parameters for the detection (see Guralnik & Srivasta, 1999)
        )
        self.nb_data_points = nb_data_points
        self.Ne = Ne
        self.df = df  # For phase correction
        self.DP = DP  # Phase increase compared to quiet
        self.DA = DA  # Amplitude increase compared to quiet
        self.p1 = p1  # Last breakpoint slope
        self.quiet = quiet  # Is 1 if it is quiet
        self.flag = flag  # Is 1 if the station is transmitting
        self.mu = mu  # Average of the flux pdf
        self.sigma = sigma  # Std of the flux pdf

    def update_df(self, this_receiver, path_to_data, epsilon=5, nb_days=30):
        """Updates the value of df for the station, as it depends on the transmitter and may vary in time

        :param receiver: Receiver class instance
        :param path_to_data: Path to the data files
        :param epsilon: Maximum phase jump (in °) that we allow on average (default: 5)
        :param nb_days: Number of days on which we want to compute df (default: 30)"""

        # Get today's date, date_inf (yesterday - nb_days) and date_sup (yesterday)
        today = dt.datetime.now(
            dt.timezone.utc
        )  # The time zone is necessary to use get_sza later
        date_sup = today - dt.timedelta(days=1)
        date_inf = date_sup - dt.timedelta(days=nb_days)

        # Compute the new df
        df = correct_phase.compute_df(
            date_inf, date_sup, self, path_to_data, this_receiver.file_endings, epsilon
        )

        # Update the df value
        self.df = df

    def update_breakpoints(
        self,
        path_breakpoint,
        path_mau,
        path_sunrise,
        alerts_antenna,
        alerts_detection,
        this_receiver,
    ):
        """Checks the phase and detects perturbations due to flares. If needed, send alerts to users

        :param path_breakpoint: path to the files where the breakpoints are stores
        :param path_mau: path to the data being written
        :param path_sunrise: path to the files with precomputed sunrise and sunset times
        :param alerts_antenna: instance of 'alert' class, alert to send if the antenna is down
        :param alerts_detection: instance of 'alert' class, alert to send if there is a detection
        :param this_receiver: Receiver class instance
        """

        try:
            # Get today's date in a good format
            today = dt.datetime.now(
                dt.timezone.utc
            )  # The time zone is necessary to use get_sza later
            la_date = today.strftime("%Y_%m_%d")

            # Read data & corrects the phase
            amp_NS, amp_EW, phase_NS, phase_EW, the_time, nb_error = (
                rf.read_Narrowband_real_time(self, path_mau, this_receiver, today)
            )

            # Before v1.3.3.post5
            phase = polar.EllipseParams(amp_NS, amp_EW, phase_NS, phase_EW)
            phase, jumps = correct_phase.correctionphaseparDw(self, phase, the_time)

            # Flag the data if needed
            # CAREFUL: amp is the median of the amplitude recorded over the last minute at most
            path_sunrise = path_sunrise + self.name + "_sunrise_sunset.txt"
            flag, nb_data_points, amp = fl.flags(
                amp_NS,
                amp_EW,
                self.nb_data_points,
                today,
                path_sunrise,
                this_receiver,
                phase,
            )
            self.nb_data_points = nb_data_points
            self.flag = flag

            if flag == 1:  # The antenna has stopped
                self.nb_error += 1
                if alerts_antenna.sent_last_time == 0 and self.nb_error == 5:
                    alerts_antenna.send_email()

            elif flag == 0:

                this_receiver.nb_error = 0
                alerts_antenna.sent_last_time = 1

                # Time average the data & apply median filter
                time_average = np.min([self.time_average, len(the_time)])
                the_time = sliding_median(the_time, self.time_average)
                the_time = the_time[0::time_average]
                phase = sliding_median(phase, self.time_average)
                phase = phase[0::time_average]

                # Read the last breakpoint value OR create a new file if it is the first of the day
                last_breakpoint, last_p1, last_amp, last_phase, quiet, DP = (
                    msf.get_lastbreakpoint(self.name, la_date, path_breakpoint)
                )

                # Keep the data after last breakpoint
                mask_time = the_time > last_breakpoint
                the_time = the_time[mask_time]
                phase = phase[mask_time]

                # Shorten the data if it is too long
                # Also consider that it is quiet time, as there would have been more changes if it was flare time
                if np.max(the_time) - the_time[0] > 2:
                    two_hours = int(
                        np.floor(7200 / self.time_average)
                    )  # Number of indices in those two hours
                    phase = phase[-1 - two_hours :]
                    the_time = the_time[-1 - two_hours :]

                    # Look if there are new breakpoints now
                    detec, p1 = dcp.find_break_point_realtime(
                        the_time, phase, self.delta
                    )

                    # NOTE: Here, the p1 value is the one corresponding to the last nine minutes of data

                    # Write new breakpoint in file
                    msf.write_newbreakpoint(
                        self.name,
                        la_date,
                        path_breakpoint,
                        the_time[-1],
                        p1,
                        amp,
                        phase[-1],
                        1,
                        0,
                    )

                    print("New breakpoint, after two quiet hours")

                    # Change the station's characteritics
                    self.quiet = 1
                    self.DA = 0
                    self.DP = 0
                    self.p1 = p1

                else:
                    # Look if there are new breakpoints now
                    detec, p1 = dcp.find_break_point_realtime(
                        the_time, phase, self.delta
                    )

                    # If there is a new breakpoint, write it in a file
                    if detec:

                        # Get last QUIET breakpoint
                        (
                            quiet_breakpoint,
                            quiet_p1,
                            quiet_amp,
                            quiet_phase,
                            quiet_quiet,
                            quiet_DP,
                        ) = msf.get_lastbreakpoint(
                            self.name, la_date, path_breakpoint, quiet=True
                        )
                        # NOTE: quiet_quiet is useless and is always 1 (same for quiet_DP who is always 0)

                        quiet_now, prev_timedecay, DP, DA = acp.analyse_breakpoint(
                            the_time[-1],
                            p1,
                            amp,
                            phase,
                            the_time,
                            quiet,
                            quiet_phase,
                            quiet_amp,
                            last_breakpoint,
                            last_p1,
                            self.detection_threshold,
                            quiet_p1,
                            quiet_breakpoint,
                        )

                        if quiet_now == 1:
                            DP = 0
                            DA = 0

                        # Write new breakpoint in file
                        msf.write_newbreakpoint(
                            self.name,
                            la_date,
                            path_breakpoint,
                            the_time[-1],
                            p1,
                            amp,
                            phase[-1],
                            quiet_now,
                            DP,
                        )

                        print("New breakpoint !")

                        # Update the station
                        self.p1 = p1
                        self.quiet = quiet_now
                        self.DP = DP
                        self.DA = DA

                        # Send alerts if needed
                        if quiet_now == 0 and quiet == 1:

                            alerts_detection.update_detection_body(self.name, p1)
                            alerts_detection.send_email()

                    print("Done - ", today.strftime("%H:%M:%S"), "-", self.name)

            elif flag == 3:

                print(self.name, " at ", today.strftime("%H:%M:%S"), " : nighttime")
                self.quiet = 1
                self.DP = 0
                self.DA = 0
                this_receiver.nb_error = 0
                alerts_antenna.sent_last_time = 1

            else:

                print(self.name, " is not transmitting at ", today.strftime("%H:%M:%S"))
                self.quiet = 1
                alerts_antenna.sent_last_time = 1

        except Exception as e:  # If there was an error, we update the log file

            this_receiver.nb_error += 1

            # Configuration of the logging file
            logging.basicConfig(filename="detection.log", encoding="utf-8")
            logging.error("Error at %s", "division", exc_info=e)
            print(self.name, " - error at ", today.strftime("%H:%M:%S"), " - see log")

    def get_pdf_coeff(self, path_to_probas):
        """Returns the mu and sigma coefficients, defining a gaussian probabiility
        distribution function associated to a station and a value of DP

        :param path_to_probas: path to the files containing the mu and sigma values
            for the porbability computations"""

        if self.DP < 0:
            self.DP = 0

        if (
            self.DP > 5
        ):  # Only do it for high DP, the other ones may be due to small noise

            probas = pd.read_csv(path_to_probas + self.name + "_probas.csv")

            # Get the correct row
            rows = probas[probas["Low"] <= self.DP]
            row = rows.tail(1)
            self.mu = row["mu"].iloc[0]
            self.sigma = row["sigma"].iloc[0]

        else:  # This is quiet time, we return the quiet proba centered in C1

            self.mu = -6  # log(1e-6)
            self.sigma = 1
